var margin = {top: 10, right: 44, bottom: 50, left: 58},
    width = 800 - margin.left - margin.right,
    height = 420 - margin.top - margin.bottom;

var x = d3.scale.log()
    .domain([100, 1e6])
    .range([0, width]);

var y = d3.scale.log()
    .domain([150, 3000])
    .range([height, 16]);

var color = d3.scale.category10().domain([0,1,2,3]);
var colorind = (_,i) => color([1,3,0,2][i]);

var xAxis = d3.svg.axis()
    .scale(x)
    .orient("bottom")
    .ticks(0, n=>n<1e4 ? n : n.toExponential().replace("+",""));

var yAxis = d3.svg.axis()
    .scale(y)
    .orient("left")
    .ticks(6, "1")
    .tickValues([150,500,1000,1500,2000,2500,3000]);

var line = d3.svg.line();

var svg = d3.select("#svg").append("g")
    .attr("transform", "scale(2)translate(" + margin.left + "," + margin.top + ")");

svg.append("defs").append("clipPath")
    .attr("id", "clip")
  .append("rect")
    .attr("width", width)
    .attr("height", height);

svg.append("g")
    .attr("class", "x axis")
    .attr("transform", "translate(0," + height + ")")
    .call(xAxis);

svg.append("g")
    .attr("class", "y axis")
    .call(yAxis);

svg.append("text")
    .attr({x: width/2, y: height+margin.bottom-14})
    .style({"text-anchor":"middle", "font-size":"14px"})
    .text("Length of string");
svg.append("text")
    .attr({"transform":"rotate(-90)", x:20-height/2, y:-margin.left, dy:"1em"})
    .style({"text-anchor":"middle", "font-size":"14px"})
    .text("Speed (MB/s)");

var paths, legend;
d3.tsv("compare.tsv", d=>d, function(error, data) {
    var names = d3.keys(data[0]).slice(1),
        n = names.length;
  
    var series = names.map(key => data.map(d => [x(d.x), y(d[key])]));
  
    paths = svg.append("g")
        .attr({"class":"lines", "clip-path":"url(#clip)"})
        .selectAll("path").data(series)
        .enter().append("path")
        .style("stroke", colorind)
        .attr("d", line);
  
    var offset = [-10, 10, 10, 9, -13];
    legend = svg.append("g").attr("class", "legend")
        .selectAll("text").data(names)
        .enter().append("text")
        .attr({dy: ".38em", x: width-3, "font-size":"16px",
               y: (d,i)=>offset[i]+(s=>s[s.length-1])(series[i])[1]})
        .style({"font-size": "16px", "text-anchor": "end"})
        .text(function(d) { return d; });
//      .attr("transform", (d,i) => "translate("+(width-200)+","+(height-5-(n-i)*25)+")");

//  legend = svg.append("g").attr("class", "legend")
//      .selectAll("g").data(names)
//      .enter().append("g")
//      .attr("transform", (d,i) => "translate("+(width-200)+","+(height-5-(n-i)*25)+")");
//
//  legend.append("line")
//      .style("stroke", colorind)
//      .attr("x2", 20);
//
//  legend.append("text")
//      .attr({dy: ".38em", x: 26, "font-size":"16px"})
//      .style("font-size", "16px")
//      .text(function(d) { return d; });

    [paths, legend].forEach(function (e) {
        e.filter((_,j)=>j>0).style("opacity", 0);
    });
});

function fadein(i) {
    [paths, legend].forEach(function (e) {
        e.filter((_,j)=>j===i)
            .transition().duration(800).ease("linear")
            .style("opacity", 1);
    });
}


var step = 0;
document.onclick = function() {
    if (++step < 5) fadein(step);
}
